<?php

namespace Drp\UserManagement\Contracts;

use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Contracts\Auth\CanResetPassword as CanResetPasswordContract;
use Illuminate\Support\Collection;

interface ManagedUser extends AuthenticatableContract, CanResetPasswordContract, AuthorizableContract
{
    /**
     * @param string|array $roles
     */
    public function usersByRoles($roles): Collection;

    public function allAdmins(): Collection;

    public function allSuperAdmins(): Collection;

    public function hasAdminAccess(): bool;

    public function hasSuperAdminAccess(): bool;

    public function canManageAdmins(): bool;

    public function hasRole(string $role): bool;

    /**
     * @param string|array $roles
     */
    public function hasAnyRole($roles): bool;

    public function getByFirstLoginToken(string $token): ManagedUser;

    public function allAwaitingApproval(): Collection;

    public function requiresApproving(): bool;

    public function approve(): void;

    public function removeFirstLoginToken(): ManagedUser;

    public function addLoginToken(string $token, \DateTime $expiry): ManagedUser;

    public function loginToken(): ?string;

    public function loginExpiry(): ?\DateTime;
}
